#!/usr/bin/env python

import os
import subprocess

filepath = os.path.realpath(__file__)
testroot = os.path.dirname(filepath)

def run(cmdline, cwd=testroot):
    subprocess.check_call(cmdline, cwd=cwd)

def create_file(filepath, content, mode='w'):
    file = open(filepath, mode)
    file.write(content)
    file.close()

def append_text(filepath, content):
    create_file(filepath, content, 'a+')

def prepend_text(filepath, content):
    file = open(filepath, 'r+')
    lines = file.readlines()
    file.seek(0)
    file.write(content)
    file.writelines(lines)
    file.close()

# tag repo
reponame = 'tagrepo'
run('hg init %s' % reponame)

repopath = os.path.join(testroot, reponame)
hoge = os.path.join(repopath, 'hoge.txt')
create_file(hoge, """TortoiseHg is a Windows shell extension and a series of
applications for the Mercurial distributed revision control system.
It also includes a Gnome/Nautilus extension and a CLI wrapper application
so the TortoiseHg tools can be used on non-Windows platforms.""")

run('hg commit --addremove -m "initial commit"', repopath)
run('hg tag normal-1', repopath)
run('hg tag normal-2', repopath)

append_text(hoge, '\n\n[From TortoiseHg Official Website]')

run('hg commit -m "add source description of test"', repopath)
run('hg tag --local -r 2 local-1', repopath)

# normal repo
reponame = 'normalrepo'
run('hg init %s' % reponame)

repopath = os.path.join(testroot, reponame)
hoge = os.path.join(repopath, 'hoge.txt')
create_file(hoge, 'something, something, something\n')
run('hg commit --addremove -m "initial commit"', repopath)

append_text(hoge, 'hogehogehoge\n\nhagehagehage\n')
run('hg commit --addremove -m "append text"', repopath)

prepend_text(hoge, 'TortoiseHg UI Test Framework\n\n')
append_text(hoge, 'TortoiseHg\n')

# mq repo
reponame = 'mqrepo'
run('hg init %s' % reponame)

repopath = os.path.join(testroot, reponame)
hoge = os.path.join(repopath, 'hoge.txt')
create_file(hoge, 'something, something, something\n')
run('hg commit --addremove -m "initial commit"', repopath)

append_text(hoge, 'hogehogehoge\n\nhagehagehage\n')
run('hg commit -m "append text"', repopath)

prepend_text(hoge, 'TortoiseHg UI Test Framework\n\n')
append_text(hoge, 'TortoiseHg\n')
run('hg --config "extensions.mq=" qnew -m "a new patch #1" patch-1', repopath)
