import gtk
import inspect

def print_result(results):
    mod_total = len(results)
    mod_failed = cond_total = cond_failed = 0

    print '\n[DETAILS]\n'
    for module_name, doc, clist in results:
        cur_failed = cond_failed
        print '%s (%s.py)' % (doc, module_name)
        for class_name, doc, mlist in clist:
            print '  %s: %s' % (class_name, doc)
            for method_name, doc, chunks in mlist:
                print '%s%s: %s' % (' ' * 4, method_name, doc)
                for msg, chunk in chunks:
                    cond_total += len(chunk)
                    if all([cond[0] for cond in chunk]):
                        print '%sPASSED: %s' % (' ' * 6, msg)
                    else:
                        print '->%sFAILED: %s' % (' ' * 4, msg)
                        for i, (cond, actual, expected) in enumerate(chunk):
                            if cond:
                                continue
                            cond_failed += 1
                            print '%s[%d] actual=%s, expected=%s' % \
                                  (' ' * 8, i, actual, expected)
        if cur_failed != cond_failed:
            mod_failed += 1

    print '\n[SUMMARY]\n'
    print '%d of %d modules FAILED, %d of %d checks FAILED' % \
          (mod_failed, mod_total, cond_failed, cond_total)

# helper functions for setting up fixture

def clear_text(widget):
    if hasattr(widget, 'set_text'):
        widget.set_text('')
    elif isinstance(widget, gtk.ComboBoxEntry):
        widget.child.set_text('')
    elif isinstance(widget, gtk.TextView):
        widget.get_buffer().set_text('')
    else:
        raise TypeError('unsupported widget: %s' % widget)

def set_active(widget, state=True):
    if hasattr(widget, 'set_active'):
        widget.set_active(state)
    else:
        raise TypeError('unsupported widget: %s' % widget)

def set_inactive(widget):
    set_active(widget, False)

# base class for gtk testcase

class GTKTestCase(object):
    def __init__(self):
        self.results = []

    ### functions to be overridden

    def setup_gtk(self, fixture):
        pass

    def teardown_gtk(self, fixture):
        pass

    ### functions used by test runner

    def set_context(self, name, doc):
        result = (name, doc, [])
        self.results.append(result)

    def get_results(self):
        return self.results

    ### assertion functions used by testcase classes

    def check(self, actual, expected, msg=None):
        all_chunks = self.results[-1][2]
        if msg is None:
            chunk = all_chunks[-1][1]
        else:
            chunk = []
            all_chunks.append((msg, chunk))
        chunk.append((actual == expected, actual, expected))

    def activated(self, widget, msg=None):
        self.check(widget.get_active(), True, msg)

    def inactivated(self, widget, msg=None):
        self.check(widget.get_active(), False, msg)

    def enabled(self, widget, msg=None):
        self.check(widget.get_property('sensitive'), True, msg)

    def disabled(self, widget, msg=None):
        self.check(widget.get_property('sensitive'), False, msg)

    def has_text(self, widget, expected, msg=None):
        types = inspect.getmro(type(widget))
        if hasattr(widget, 'get_text'):
            actual = widget.get_text()
        elif isinstance(widget, gtk.ComboBoxEntry):
            actual = widget.child.get_text()
        elif isinstance(widget, gtk.TextView):
            buf = widget.get_buffer()
            actual = buf.get_text(buf.get_start_iter(),
                                  buf.get_end_iter())
        elif gtk.Button in types or gtk.ToolButton in types:
            actual = widget.get_label()
        elif gtk.Window in types:
            actual = widget.get_title()
        else:
            raise TypeError('unsupported widget: %s' % widget)
        self.check(actual, expected, msg)

    def cleared(self, widget, msg=None):
        self.has_text(widget, '', msg)
