#!/usr/bin/env python
#
# runner.py - TortoiseHg test runner
#
# Copyright 2010 Yuki KODAMA <endflow.net@gmail.com>
#
# This software may be used and distributed according to the terms of the
# GNU General Public License version 2, incorporated herein by reference.

import os
import sys
import time
import threading
import inspect
import gobject
import gtk

import gtktest

# add package path
filepath = os.path.realpath(__file__)
testpath = os.path.dirname(filepath)
thgpath = os.path.dirname(testpath)
sys.path.insert(0, thgpath)

# use 'English' UI language
os.environ['LANGUAGE'] = os.environ['LANG'] = 'en'
os.environ['HOME'] = os.path.join(testpath, 'home')

def test_runner(runner_ready, fixture_ready, holder):
    allresults = []
    alltests = [os.path.splitext(p)[0] for p in os.listdir(testpath)
                    if p.startswith('test_') and p.endswith('.py')]
    alltests.sort()
#    alltests = ['test_thgconfig']
    for modname in alltests:
        modobj = __import__(modname)

        # prepare fixture (dialog)
        gtk.gdk.threads_enter()
        fixture = modobj.create_fixture(testpath)
        gtk.gdk.threads_leave()
        holder[0] = fixture

        # notify to main: dialog is created
        runner_ready.set()

        # wait for main: setup fixture
        fixture_ready.wait()
        fixture_ready.clear()

        # wait for user-defined ready
        if hasattr(modobj, 'wait_fixture'):
            modobj.wait_fixture(fixture)
        else:
            time.sleep(1)

        clsresults = []
        for clsname, clsobj in inspect.getmembers(modobj, inspect.isclass):
            if not clsname.endswith('Test') or \
               not gtktest.GTKTestCase in inspect.getmro(clsobj):
                continue

            test = clsobj()
            for name, method in inspect.getmembers(test,
                                                   inspect.ismethod):
                if not name.startswith('test_'):
                    continue

                try:
                    gtk.gdk.threads_enter()
                    test.set_context(name, inspect.getdoc(method))
                    test.setup_gtk(fixture)
                    method(fixture)
                    test.teardown_gtk(fixture)
                    gtk.gdk.threads_leave()
                except:
                    print 'error in %s' % name, sys.exc_type, sys.exc_value

            clsresult = (clsname, inspect.getdoc(clsobj), test.get_results())
            clsresults.append(clsresult)

        gtk.main_quit()
        result = (modname, inspect.getdoc(modobj), clsresults)
        allresults.append(result)

    # notify to main: no more tests
    holder[0] = None
    runner_ready.set()

    # output results
    gtktest.print_result(allresults)

# create event objects
runner_ready = threading.Event()
fixture_ready = threading.Event()

# start thread for test runner
holder = [None] # fixture
th = threading.Thread(target=test_runner,
                      args=[runner_ready, fixture_ready, holder])
th.start()

# setup loop for gtk dialogs
while True:
    # wait for runner: dialog is created
    runner_ready.wait()
    runner_ready.clear()
    dlg = holder[0]
    if dlg is None:
        break

    gtk.gdk.threads_init()
    gtk.gdk.threads_enter()

    # setup fixture (gtk dialog)
    if hasattr(dlg, 'display'):
        dlg.display()
    dlg.show_all()
    if 'response' in gobject.signal_list_names(dlg):
        dlg.connect('response', gtk.main_quit)
    dlg.connect('destroy', gtk.main_quit)

    # notify to runner: dialog is ready
    fixture_ready.set()

    gtk.main()
    gtk.gdk.threads_leave()
