"""GCommit test"""

import os
import gtk
import gobject
import threading

from gtktest import GTKTestCase
from gtktest import clear_text, set_active, set_inactive

def create_fixture(testroot):
    from mercurial import hg, ui
    u = ui.ui()
    repopath = os.path.join(testroot, 'normalrepo')
    repo = hg.repository(u, path=repopath)

    opts = {}
    cmdoptions = {
        'user':opts.get('user', ''), 'date':opts.get('date', ''),
        'logfile':'', 'message':'',
        'modified':True, 'added':True, 'removed':True, 'deleted':True,
        'unknown':True, 'ignored':False, 'subrepo':True,
        'exclude':[], 'include':[], 'rev':[],
        'check': True, 'git':False, 'addremove':False,
    }

    from tortoisehg.hgtk import commit
    return commit.GCommit(u, repo, repopath, [], cmdoptions)

def wait_fixture(dlg):
    ev = threading.Event()

    orig_func = dlg.refresh_complete
    def hook_func():
        orig_func()
        ev.set()
    dlg.refresh_complete = hook_func

    while not ev.is_set():
        ev.wait(0.2)
    dlg.refresh_complete = orig_func

class CommitTestBase(GTKTestCase):
    def setup_gtk(self, dlg):
        clear_text(dlg.text)

class ModeTest(CommitTestBase):
    """mode transition checks"""

    def test_commitmode(self, dlg):
        """commit mode"""
        self.has_text(dlg, 'normalrepo - commit', 'check')
        self.has_text(dlg.get_toolbutton('commit'), 'Commit')
        self.enabled(dlg.branchbutton)
        self.has_text(dlg.branchbutton, 'branch: default')
        self.cleared(dlg.text)
        self.check(dlg.get_preview_tab_name(), 'Commit Preview')

    def test_qnewmode(self, dlg):
        """qnew mode"""
        self.cleared(dlg.qnew_name, 'preconditions')
        self.has_text(dlg, 'normalrepo - commit')
        self.has_text(dlg.get_toolbutton('commit'), 'Commit')
        self.enabled(dlg.branchbutton)
        self.cleared(dlg.text)
        self.check(dlg.get_preview_tab_name(), 'Commit Preview')

        ev = threading.Event()

        orig = dlg.refresh_complete
        def refresh_complete():
            orig()
            ev.set()
        dlg.refresh_complete = refresh_complete

        def set_qnew_name(name):
            dlg.qnew_name.set_text(name)
            return False
        gobject.idle_add(set_qnew_name, 'patch')
        while not ev.is_set():
            ev.wait(0.2)
            gtk.main_iteration(False)
        ev.clear()

        self.has_text(dlg, 'normalrepo - qnew', 'entered QNew mode')
        self.has_text(dlg.get_toolbutton('commit'), 'QNew')
        self.disabled(dlg.branchbutton)
        self.cleared(dlg.text)
        self.check(dlg.get_preview_tab_name(), 'Patch Preview')

        gobject.idle_add(set_qnew_name, '')
        while not ev.is_set():
            ev.wait(0.2)
            gtk.main_iteration(False)
        ev.clear()

        self.cleared(dlg.qnew_name, 'backed to Commit mode')
        self.has_text(dlg, 'normalrepo - commit')
        self.has_text(dlg.get_toolbutton('commit'), 'Commit')
        self.enabled(dlg.branchbutton)
        self.cleared(dlg.text)
        self.check(dlg.get_preview_tab_name(), 'Commit Preview')