"""GCommit with MQ test"""

import os
import gtk
import time
import gobject
import threading

from gtktest import GTKTestCase
from gtktest import clear_text, set_active, set_inactive

def create_fixture(testroot):
    from mercurial import hg, ui
    u = ui.ui()
    repopath = os.path.join(testroot, 'mqrepo')
    repo = hg.repository(u, path=repopath)

    opts = {}
    cmdoptions = {
        'user':opts.get('user', ''), 'date':opts.get('date', ''),
        'logfile':'', 'message':'',
        'modified':True, 'added':True, 'removed':True, 'deleted':True,
        'unknown':True, 'ignored':False, 'subrepo':True,
        'exclude':[], 'include':[], 'rev':[],
        'check': True, 'git':False, 'addremove':False,
    }

    from tortoisehg.hgtk import commit
    return commit.GCommit(u, repo, repopath, [], cmdoptions)

def wait_fixture(dlg):
    ev = threading.Event()

    orig_func = dlg.refresh_complete
    def hook_func():
        orig_func()
        ev.set()
    dlg.refresh_complete = hook_func

    while not ev.is_set():
        ev.wait(0.2)
    dlg.refresh_complete = orig_func

ev = None
def wait_reload(dlg):
    global ev
    if ev is None:
        ev = threading.Event()

        orig_func = dlg.refresh_complete
        def hook_func():
            orig_func()
            ev.set()
        dlg.refresh_complete = hook_func

    while not ev.is_set():
        ev.wait(0.2)
        gtk.main_iteration(False)
    ev.clear()

class ModeTest(GTKTestCase):
    """mode transition checks with MQ"""

    def test_qrefreshmode(self, dlg):
        """qrefresh mode"""
        self.has_text(dlg, 'mqrepo - qrefresh patch-1', 'check')
        self.has_text(dlg.get_toolbutton('commit'), 'QRefresh')
        self.disabled(dlg.branchbutton)
        self.has_text(dlg.text, 'a new patch #1')
        self.check(dlg.get_preview_tab_name(), 'Patch Preview')

    def test_qnewmode(self, dlg):
        """qnew mode"""
        self.cleared(dlg.qnew_name, 'preconditions')
        self.has_text(dlg.get_toolbutton('commit'), 'QRefresh')
        self.has_text(dlg, 'mqrepo - qrefresh patch-1')
        self.disabled(dlg.branchbutton)
        self.has_text(dlg.text, 'a new patch #1')
        self.check(dlg.get_preview_tab_name(), 'Patch Preview')

        def set_qnew_name(name):
            dlg.qnew_name.set_text(name)
            return False
        gobject.idle_add(set_qnew_name, 'patch-2')
        wait_reload(dlg)

        self.has_text(dlg, 'mqrepo - qnew', 'entered QNew mode')
        self.has_text(dlg.get_toolbutton('commit'), 'QNew')
        self.disabled(dlg.branchbutton)
        self.cleared(dlg.text)
        self.check(dlg.get_preview_tab_name(), 'Patch Preview')

        gobject.idle_add(set_qnew_name, '')
        wait_reload(dlg)

        self.cleared(dlg.qnew_name, 'backed to QRefresh mode')
        self.has_text(dlg.get_toolbutton('commit'), 'QRefresh')
        self.has_text(dlg, 'mqrepo - qrefresh patch-1')
        self.disabled(dlg.branchbutton)
        self.has_text(dlg.text, 'a new patch #1')
        self.check(dlg.get_preview_tab_name(), 'Patch Preview')