"""TagAddDialog test"""

import os
import gtk
import time

from gtktest import GTKTestCase
from gtktest import clear_text, set_active, set_inactive

def create_fixture(testroot):
    from mercurial import hg, ui
    u = ui.ui()
    repo = hg.repository(u, path=os.path.join(testroot, 'tagrepo'))

    from tortoisehg.hgtk import tagadd
    return tagadd.TagAddDialog(repo)

class TagAddTestBase(GTKTestCase):
    def setup_gtk(self, dlg):
        clear_text(dlg.tagcombo)
        clear_text(dlg.reventry)

        set_inactive(dlg.localchk)
        set_inactive(dlg.engchk)
        set_inactive(dlg.replacechk)
        set_inactive(dlg.customchk)

    def teardown_gtk(self, dlg):
        set_active(dlg.engchk)

class StartupTest(TagAddTestBase):
    """initial status checks"""

    def test_notag(self, dlg):
        """started with no tag"""
        dlg.reventry.set_text('3')

        self.cleared(dlg.tagentry, 'start with no tag')
        self.enabled(dlg.localchk)
        self.inactivated(dlg.localchk)
        self.enabled(dlg.replacechk)
        self.inactivated(dlg.replacechk)
        self.enabled(dlg.engchk)
        self.inactivated(dlg.engchk)
        self.enabled(dlg.customchk)
        self.inactivated(dlg.customchk)
        self.disabled(dlg.msgentry)
        self.disabled(dlg.addbtn)
        self.disabled(dlg.removebtn)

    def test_normaltag(self, dlg):
        """started with tag"""
        dlg.tagentry.set_text('normal-1')

        self.has_text(dlg.reventry, '0', 'start with "normal-1" tag')
        self.enabled(dlg.localchk)
        self.inactivated(dlg.localchk)
        self.enabled(dlg.replacechk)
        self.inactivated(dlg.replacechk)
        self.enabled(dlg.engchk)
        self.inactivated(dlg.engchk)
        self.enabled(dlg.customchk)
        self.inactivated(dlg.customchk)
        self.disabled(dlg.msgentry)
        self.disabled(dlg.addbtn)
        self.enabled(dlg.removebtn)

    def test_localtag(self, dlg):
        """started with local tag"""
        dlg.tagentry.set_text('local-1')

        self.has_text(dlg.reventry, '2', 'start with "local-1" tag')
        self.enabled(dlg.localchk)
        self.activated(dlg.localchk)
        self.enabled(dlg.replacechk)
        self.inactivated(dlg.replacechk)
        self.disabled(dlg.engchk)
        self.inactivated(dlg.engchk)
        self.disabled(dlg.customchk)
        self.inactivated(dlg.customchk)
        self.disabled(dlg.msgentry)
        self.disabled(dlg.addbtn)
        self.enabled(dlg.removebtn)

class BasicTest(TagAddTestBase):
    """basic behaviors"""

    def test_buttons1(self, dlg):
        """button sensitives #1"""
        self.disabled(dlg.addbtn, 'preconditions')
        self.disabled(dlg.removebtn)

        dlg.tagentry.set_text('hogehoge')

        self.disabled(dlg.addbtn, 'still disabled')
        self.disabled(dlg.removebtn)

        clear_text(dlg.tagcombo)
        dlg.reventry.set_text('0')

        self.disabled(dlg.addbtn, 'still disabled')
        self.disabled(dlg.removebtn)

        dlg.tagentry.set_text('hoge')
        dlg.reventry.set_text('0')

        self.enabled(dlg.addbtn, 'enabled')
        self.disabled(dlg.removebtn)

    def test_buttons2(self, dlg):
        """button sensitives #2"""
        self.disabled(dlg.addbtn, 'preconditions')
        self.disabled(dlg.removebtn)

        dlg.tagentry.set_text('normal-1')

        self.disabled(dlg.addbtn, 'existing tag name')
        self.enabled(dlg.removebtn)

        dlg.tagentry.set_text('normal-3')

        self.enabled(dlg.addbtn, 'new tag name')
        self.disabled(dlg.removebtn)

    def test_revnum(self, dlg):
        """rev number changed"""
        self.disabled(dlg.addbtn, 'preconditions')
        self.disabled(dlg.removebtn)

        dlg.tagentry.set_text('normal-2')

        self.has_text(dlg.reventry, '1', '"normal-2" tag is selected')
        self.enabled(dlg.localchk)
        self.inactivated(dlg.localchk)
        self.enabled(dlg.replacechk)
        self.inactivated(dlg.replacechk)
        self.enabled(dlg.engchk)
        self.inactivated(dlg.engchk)
        self.enabled(dlg.customchk)
        self.inactivated(dlg.customchk)
        self.disabled(dlg.msgentry)
        self.disabled(dlg.addbtn)
        self.enabled(dlg.removebtn)

        dlg.reventry.set_text('100')

        self.disabled(dlg.addbtn, "revision 100 doesn't exist")
        self.disabled(dlg.removebtn)

        dlg.reventry.set_text('2')

        self.disabled(dlg.addbtn, "revision 2 doesn't have 'normal-2' tag")
        self.disabled(dlg.removebtn)

        dlg.reventry.set_text('1')

        self.enabled(dlg.addbtn, "revision 1 has 'normal-2' tag")
        self.disabled(dlg.removebtn)

class OptionTest(TagAddTestBase):
    """option behaviors"""

    def test_localchk1(self, dlg):
        """local tag check button #1: normal path"""
        self.inactivated(dlg.localchk, 'preconditions')
        self.enabled(dlg.engchk)
        self.enabled(dlg.customchk)
        self.disabled(dlg.msgentry)

        set_active(dlg.localchk)

        self.activated(dlg.localchk, 'check')
        self.disabled(dlg.engchk)
        self.disabled(dlg.customchk)
        self.disabled(dlg.msgentry)

    def test_localchk2(self, dlg):
        """local tag check button #2: local option and activated tag"""
        self.inactivated(dlg.localchk, 'preconditions')
        self.disabled(dlg.addbtn)
        self.disabled(dlg.removebtn)

        dlg.tagentry.set_text('local-1')

        self.activated(dlg.localchk, 'should activate local tag option')
        self.disabled(dlg.addbtn)
        self.enabled(dlg.removebtn)

        dlg.tagentry.set_text('local-3')

        self.activated(dlg.localchk, 'should keep local tag option')
        self.enabled(dlg.addbtn)
        self.disabled(dlg.removebtn)

        dlg.tagentry.set_text('normal-1')

        self.inactivated(dlg.localchk, 'should inactivate local tag option')
        self.disabled(dlg.addbtn)
        self.enabled(dlg.removebtn)

    def test_localchk3(self, dlg):
        """local tag check button #3: local option, rev num and buttons"""
        dlg.tagentry.set_text('normal-1')

        self.has_text(dlg.reventry, '0', '"normal-1" tag can be removed')
        self.inactivated(dlg.localchk)
        self.disabled(dlg.addbtn)
        self.enabled(dlg.removebtn)

        set_active(dlg.localchk)

        self.disabled(dlg.addbtn, '"normal-1" is normal tag, not local tag')
        self.disabled(dlg.removebtn)

        dlg.tagentry.set_text('local-1')

        self.has_text(dlg.reventry, '2', '"local-1" tag can be removed')
        self.activated(dlg.localchk)
        self.disabled(dlg.addbtn)
        self.enabled(dlg.removebtn)

        set_inactive(dlg.localchk)

        self.disabled(dlg.addbtn, '"local-1" is local tag, not normal tag')
        self.disabled(dlg.removebtn)

    def test_customchk(self, dlg):
        """custom message check button"""
        self.inactivated(dlg.customchk, 'preconditions')
        self.disabled(dlg.msgentry)

        dlg.customchk.set_active(True)

        self.activated(dlg.customchk, 'check')
        self.enabled(dlg.msgentry)

    def test_replacechk(self, dlg):
        """tag replacement check button"""
        self.inactivated(dlg.replacechk, 'preconditions')
        self.disabled(dlg.addbtn)
        self.disabled(dlg.removebtn)

        dlg.tagentry.set_text('normal-1')

        self.disabled(dlg.addbtn, 'should keep disabled')
        self.enabled(dlg.removebtn)

        set_active(dlg.replacechk)

        self.enabled(dlg.addbtn, 'should change to enabled')
        self.enabled(dlg.removebtn)